(function ($, Drupal, once, drupalSettings) {
  Drupal.behaviors.domPathFinder = {
    attach: function (context, settings) {
      const getElementPath = (element) => {
        let path = [];
        while (element && element !== document.body) {
          let selector = '';

          if (element.getAttribute('data-drupal-selector')) {
            selector = `[data-drupal-selector="${element.getAttribute('data-drupal-selector')}"]`;
          }

          else if (element.classList.contains('field--item')) {
            const fieldItems = element.closest('.field--items');
            if (fieldItems) {
              const items = Array.from(fieldItems.getElementsByClassName('field--item'));
              const index = items.indexOf(element) + 1;
              selector = `.field--item:nth-child(${index})`;
            }
          }
          else if (element.hasAttribute('data-entity-uuid')) {
            selector = `[data-entity-uuid="${element.getAttribute('data-entity-uuid')}"]`;
          }
          else {
            const drupalClasses = Array.from(element.classList)
              .filter(cls =>
                cls.startsWith('field--') ||
                cls.startsWith('node--') ||
                cls.startsWith('block--') ||
                cls.startsWith('paragraph--')
              );

            if (drupalClasses.length) {
              selector = `.${drupalClasses.join('.')}`;
            } else {
              if (element.id) {
                selector = `#${element.id}`;
              } else {
                const tagName = element.tagName.toLowerCase();
                const siblings = element.parentNode ?
                  Array.from(element.parentNode.children).filter(child =>
                    child.tagName === element.tagName
                  ) : [];
                const index = siblings.indexOf(element) + 1;
                selector = siblings.length > 1 ?
                  `${tagName}:nth-of-type(${index})` : tagName;
              }
            }
          }

          path.unshift(selector);
          element = element.parentNode;
        }

        return path.join(' > ');
      };


      const isValidTarget = (element) => {
        const classes = Array.from(element.classList);
        return !classes.some(className =>
          className.startsWith('todo-') ||
          className.startsWith('form-')
        );
      };

      once('domPathFinder', '.add-pin', context).forEach((button) => {
        button.addEventListener('click', async function(e) {
          e.preventDefault();
          document.querySelector('.contrib-todo-list').classList.remove('show');
          const todoId = this.dataset.todoId;

          document.body.classList.add('pin-selection-mode');
          document.body.style.cursor = 'crosshair';

          const handleDocumentClick = async function(event) {
            event.preventDefault();

            if (!isValidTarget(event.target)) {
              document.body.classList.remove('pin-selection-mode');
              document.body.style.cursor = 'default';
              document.removeEventListener('click', handleDocumentClick);
              const messages = new Drupal.Message();
              messages.add(
                Drupal.t('Cannot pin elements with classes starting with "todo-" or "form-".'),
                {type: 'error'}
              );
              return;
            }

            const pin = getElementPath(event.target);

            document.body.classList.remove('pin-selection-mode');
            document.body.style.cursor = 'default';

            try {
              const response = await fetch(`/todo/${todoId}/pin`, {
                method: 'POST',
                headers: {
                  'Content-Type': 'application/json',
                },
                body: JSON.stringify({ pin: pin })
              });

              if (response.ok) {
                Drupal.behaviors.todoListPins.addPinToElement(pin, todoId);          
                document.querySelector('.contrib-todo-list').classList.add('show');
              }
            } catch (error) {}

            document.removeEventListener('click', handleDocumentClick);
          };

          requestAnimationFrame(() => {
            document.addEventListener('click', handleDocumentClick, { once: true });
          });
        });
      });
    }
  };



  Drupal.behaviors.todoListPins = {
    addPinToElement: function(pin, todoId) {
      const targetElement = document.querySelector(pin);
      if (targetElement) {
        if (!document.querySelector(`#todo-pin-${todoId}`)) {
          const pinElement = document.createElement('span');
          pinElement.innerHTML = '📌';
          pinElement.id = `todo-pin-${todoId}`;
          pinElement.className = 'todo-pin';
          targetElement.appendChild(pinElement);

          const addPinButton = document.querySelector(`.add-pin[data-todo-id="${todoId}"]`);
          if (addPinButton) {
            const goToPinDiv = document.createElement('div');
            goToPinDiv.className = 'go-pin';
            goToPinDiv.setAttribute('title', Drupal.t('Go to pin'));
            goToPinDiv.setAttribute('data-todo-id', todoId);
            goToPinDiv.innerHTML = `<svg width="24" height="24" viewBox="-0.5 0 25 25" fill="none" xmlns="http://www.w3.org/2000/svg">
                                    <path d="M3 11.3201C3 8.93312 3.94822 6.64394 5.63605 4.95612C7.32387 3.26829 9.61305 2.32007 12 2.32007C14.3869 2.32007 16.6762 3.26829 18.364 4.95612C20.0518 6.64394 21 8.93312 21 11.3201" stroke="#000000" stroke-width="1.5" stroke-linecap="round" stroke-linejoin="round"/>
                                    <path d="M3 11.3201C3 17.4201 9.76 22.3201 12 22.3201C14.24 22.3201 21 17.4201 21 11.3201" stroke="#000000" stroke-width="1.5" stroke-linecap="round" stroke-linejoin="round"/>
                                    <path d="M12 15.3201C14.2091 15.3201 16 13.5292 16 11.3201C16 9.11093 14.2091 7.32007 12 7.32007C9.79086 7.32007 8 9.11093 8 11.3201C8 13.5292 9.79086 15.3201 12 15.3201Z" stroke="#000000" stroke-width="1.5" stroke-linecap="round" stroke-linejoin="round"/>
                                    </svg>`;

            goToPinDiv.addEventListener('click', function(e) {
              e.preventDefault();
              const pinElement = document.querySelector(`#todo-pin-${todoId}`);
              if (pinElement) {
                pinElement.scrollIntoView({
                  behavior: 'smooth',
                  block: 'center'
                });
              }
            });

            addPinButton.replaceWith(goToPinDiv);
          }
        }
      }
    },

    attach: function (context, settings) {
      const todos = drupalSettings.contrib_todo_list.todos || [];
      once('initPin', '[data-history-node-id]', context).forEach((element) => {
        setTimeout(() => {
          Object.values(todos).forEach(todo => {
            if (todo.pin) {
              this.addPinToElement(todo.pin, todo.id);
            }
          });
        }, 200);
      });

      once('pinScroll', '.go-pin', context).forEach((button) => {
        $(button).on('click', function(e) {
          e.preventDefault();
          const todoId = $(this).data('todo-id');
          const pinElement = $(`#todo-pin-${todoId}`);

          if (pinElement.length) {
            pinElement[0].scrollIntoView({
              behavior: 'smooth',
              block: 'center'
            });
          }
        });
      });
    }
  };
})(jQuery, Drupal, once, drupalSettings);
